angular.module('getlaundry.providers_controller', [])

        .controller('ProvidersCtrl', function ($scope, $state, $timeout, Session, FOrder, Message, $ionicModal, $ionicPopup, IonicClosePopupService) {

            $scope.default_img = 'img/default_item.jpg';
            var id_area = Session.get('loc_from_area');

            $scope.reload_providers = function () {

                var location = angular.fromJson(Session.get('location'));
                if (parseInt(id_area) > 0) {
                    location = {
                        manual: true,
                        id_area: id_area
                    };
                }

                FOrder.get_providers_by_location(location)
                        .success(function (response) {

                            if (response.success) {
                                Session.set('address', angular.toJson(response.address));
                                Session.set('providers', angular.toJson(response.providers));

                                $scope.providers = response.providers;
                                $scope.$broadcast('scroll.refreshComplete');
                            }
                            else {
                                if (response.errorCode == 'NO_PROVIDERS') {
                                    $scope.show_empty();
                                }
                                else {
                                    Message.show(response.error);
                                }
                            }
                        });
            };

            $scope.load_providers = function () {
                var providers = angular.fromJson(Session.get('providers'));
                if (typeof providers != 'undefined' && providers != null) {
                    $scope.providers = providers;
                    $scope.$broadcast('scroll.refreshComplete');
                }
                else {
                    $state.go('app.map');
                }
            };

            var myPopup;
            $scope.show_offers = function () {

                // Add animate css class to out popup container
                $timeout(function () {
                    var popupElements = document.getElementsByClassName("popup-container");
                    if (popupElements.length) {
                        $scope.popupElement = angular.element(popupElements[0]);
                        $scope.popupElement.addClass('animated');
                        $scope.popupElement.addClass('bounceInDown');
                    }
                }, 1);

                myPopup = $ionicPopup.show({
                    templateUrl: 'templates/offers.html',
                    title: $scope.offer.title,
                    scope: $scope
                });
                IonicClosePopupService.register(myPopup);

                try
                {
                    ga_storage._trackPageview('show_offer.html', 'Show Offer');
                }
                catch (e)
                {
                    console.log(e.message);
                }
            };

            $scope.close_offers = function () {
                myPopup.close();
            };

            $scope.check_for_offers = function () {

                var location = angular.fromJson(Session.get('location'));
                if (parseInt(id_area) > 0) {
                    location = {
                        manual: true,
                        id_area: id_area
                    };
                }

                FOrder.get_offers(location)
                        .success(function (response) {
                            if (response.success) {
                                $scope.offer = response.offer;
                                $scope.show_offers();
                            }
                        });
            };

            $scope.$on("$ionicView.enter", function (event, data) {
                FOrder.deleteProvider();
                FOrder.resetCart();
                FOrder.deleteOrder();
                Session.unset('previous_view');

                $scope.check_for_offers();

                $scope.load_providers();

                try
                {
                    ga_storage._trackPageview('providers.html', 'Providers');
                }
                catch (e)
                {
                    console.log(e.message);
                }
            });

            $scope.view_provider = function (id_provider) {
                $state.go('app.provider_details', {id_provider: id_provider});
            };

            $scope.get_suggested_provider = function () {
                var found = false;
                var id_provider = $scope.providers[0].id_provider;

                angular.forEach($scope.providers, function (value, key) {
                    if (!found) {
                        if (value.is_suggested == 1) {
                            id_provider = value.id_provider;
                            found = true;
                        }
                    }
                });

                return id_provider;
            };

            $scope.choose_laundry = function () {
                FOrder.resetCart();
                FOrder.deleteProvider();

                var id_provider = $scope.get_suggested_provider();

                try
                {
                    ga_storage._trackPageview('let_us_choose.html', 'Let us choose for you');
                }
                catch (e)
                {
                    console.log(e.message);
                }

                $state.go('app.provider_details', {id_provider: id_provider});
            };

            $ionicModal.fromTemplateUrl('templates/filter_providers.html', {
                scope: $scope
            }).then(function (modal) {
                $scope.modalFilter = modal;
            });

            // Triggered in the login modal to close it
            $scope.close_filter = function () {
                $scope.modalFilter.hide();
            };

            // Open the login modal
            $scope.show_filter = function () {
                $scope.modalFilter.show();
            };

            $scope.$on("$ionicView.beforeLeave", function (event, data) {
                if (typeof myPopup != 'undefined') {
                    myPopup.close();
                }
            });
        })

        .controller('ProviderDetailsCtrl', function ($scope, Auth, $translate, $ionicFilterBar, $timeout, $ionicPopup, IonicClosePopupService, $ionicSideMenuDelegate, Session, Push, $stateParams, $state, Message, $ionicScrollDelegate, FOrder, FCustomer) {

            $scope.loading_garments = false;
            $scope.try_again = false;
            $scope.garments = [];
            $scope.default_cover = 'img/default.jpg';
            $scope.default_item = 'img/default_item.png';
            $scope.offer_seen = false;
            var id_provider = $stateParams.id_provider;

            $scope.is_editing = parseInt(Session.get('editing_order')) > 0;

            var providers = FOrder.getProviders();

            if (typeof providers != 'undefined' && providers != null) {
                $scope.one_provider = providers.length == 1;
            }

            $ionicSideMenuDelegate.canDragContent(false);

            $scope.cart = FOrder.getCart();

            $scope.show_push = function () {
                // ASk for Push Notifications
                if (Auth.isLogged()) {
                    $scope.customer = FCustomer.getCustomer();
                    if (!$scope.customer.id_gcm) {
                        if (Session.get('push_asked') != 1) {
                            var title = $translate.instant('allow_notifications');
                            var message = $translate.instant('notification_text');

                            Session.set('push_asked', 1);

                            Message.confirm(title, message).then(function (res) {
                                if (res === true || res == 2) {
                                    Push.register();
                                }
                            });
                        }
                    }
                    else {
                        Push.register();
                    }
                }
            };

            $scope.load_provider = function () {

                var provider = FOrder.getProvider();
                if (typeof provider != 'null' && provider != null) {
                    $scope.provider = provider;
                    $scope.offer = provider.offer;
                    $scope.active_segment = provider.services[0].id_service;
                    $scope.garments = FOrder.syncCart(provider.garments);
                    $scope.active_service = $scope.cart.service;

                    $scope.$broadcast('scroll.refreshComplete');
                }
                else {
                    FOrder.get_provider(id_provider)
                            .success(function (response) {
                                if (response.success) {
                                    $scope.provider = response.provider;
                                    $scope.offer = response.provider.offer;
                                    $scope.active_segment = response.provider.services[0].id_service;
                                    $scope.garments = FOrder.syncCart(response.provider.garments);
                                    $scope.active_service = 1;

                                    FOrder.saveProvider(response.provider);

                                    $scope.$broadcast('scroll.refreshComplete');

                                    setTimeout(function () {
                                        $scope.show_push();
                                    }, 1000);

                                    try
                                    {
                                        ga_storage._trackPageview('provider.html', response.provider.name);
                                    }
                                    catch (e)
                                    {
                                        console.log(e.message);
                                    }
                                }
                            });
                }
            };

            $scope.load_provider();

            $scope.activate_segment = function (index) {
                $scope.active_segment = index;

                $ionicScrollDelegate.resize();
            };

            $scope.increment = function (garment, $event) {
                $event.stopPropagation();

                garment['items']++;
                garment['partial_amount'] = garment.items * garment.price;

                if (garment.is_kg != 1) {
                    $scope.cart.total_items++;
                }
                else {
                    $scope.cart.total_kg++;
                }

                $scope.cart.total_amount = parseFloat($scope.cart.total_amount) + parseFloat(garment.price);
                $scope.cart.garments[garment.id_garment_has_service] = garment;

                FOrder.saveCart($scope.cart);
            };

            $scope.decrement = function (garment, $event) {
                $event.stopPropagation();

                if (garment.items > 0) {
                    garment['items']--;
                    garment['partial_amount'] = garment.items * garment.price;

                    if (garment.is_kg != 1) {
                        $scope.cart.total_items--;
                    }
                    else {
                        $scope.cart.total_kg--;
                    }

                    $scope.cart.total_amount = parseFloat($scope.cart.total_amount) - parseFloat(garment.price);
                    $scope.cart.garments[garment.id_garment_has_service] = garment;

                    if (garment['items'] == 0) {
                        delete $scope.cart.garments[garment.id_garment_has_service];
                    }

                    FOrder.saveCart($scope.cart);
                }
            };

            $scope.go_schedule = function () {

                var message = '';
                var pro_has_kg = parseFloat($scope.provider.minimum_kg) > 0;
                var m_min = $translate.instant('minimum') + ' ' + $scope.provider.minimum_amount + ' ' + $scope.provider.currency;
                var m_min_kg = $translate.instant('minimum_kg') + ' ' + $scope.provider.minimum_kg + ' Kg';

                if (parseFloat($scope.cart.total_amount) == 0) {
                    if (pro_has_kg) {
                        message = m_min + '<br /><br />' + m_min_kg;
                    }
                    else {
                        message = m_min;
                    }

                    Message.alert(message);
                    $state.go('app.schedule');
                }
                else if (parseFloat($scope.cart.total_amount) < parseFloat($scope.provider.minimum_amount)) {
                    if (pro_has_kg) {
                        if (parseFloat($scope.cart.total_kg) < parseFloat($scope.provider.minimum_kg)) {
                            message = m_min + '<br /><br />' + m_min_kg;
                            Message.alert(message);
                            return false;
                        }
                        else {
                            $state.go('app.schedule');
                        }
                    }
                    else {
                        message = m_min;
                        Message.alert(message);
                        return false;
                    }
                }
                else {
                    $state.go('app.schedule');
                }
            };

            $scope.has_kg = function () {
                for (var i = 0; i < $scope.garments.length; i++) {
                    if ($scope.garments[i].id_service == 99) {
                        return true;
                    }
                }

                return false;
            };

            $scope.get_service_index = function () {
                var service = $scope.active_segment;
                var services = $scope.provider.services;
                var pos = 0;

                for (var i = 0; i < services.length; i++) {
                    if (service == services[i].id_service) {
                        pos = i;
                    }
                }

                return pos;
            };

            $scope.onSwipeLeft = function () {

                var pos = $scope.get_service_index();
                var next = parseInt(pos) + 1;

                if (typeof $scope.provider.services[next] != 'undefined') {
                    var id = $scope.provider.services[next].id_service;
                    $scope.activate_segment(id);
                }
            };

            $scope.onSwipeRight = function () {
                var pos = $scope.get_service_index();
                var prev = parseInt(pos) - 1;

                if (typeof $scope.provider.services[prev] != 'undefined') {
                    var id = $scope.provider.services[prev].id_service;
                    $scope.activate_segment(id);
                }
            };

            $scope.ask_cancel = function () {
                var title = $translate.instant('cancel_editing') + ' #' + Session.get('editing_order');
                var message = $translate.instant('stop_editing');

                Message.confirm(title, message).then(function (res) {
                    if (res === true || res == 2) {

                        FOrder.deleteProviders();
                        FOrder.deleteProvider();
                        FOrder.resetOrder();
                        FOrder.resetCart();
                        Session.unset('previous_view');
                        Session.unset('editing_order');
                        $scope.adding_address = false;
                        $scope.previous_view = '';

                        $state.go('app.map');
                    }
                });
            };

            $scope.go_map = function () {
                FOrder.deleteProviders();
                FOrder.deleteProvider();
                FOrder.resetOrder();
                FOrder.resetCart();
                Session.unset('previous_view');
                Session.unset('editing_order');

                $state.go('app.map');
            };

            var filterBarInstance;

            $scope.showFilterBar = function () {
                filterBarInstance = $ionicFilterBar.show({
                    items: $scope.garments,
                    update: function (filteredItems, filterText) {
                        if (filterText) {
                            $scope.search = filterText;
                        }
                        else {
                            $scope.search = '';
                        }
                        $scope.$broadcast('scroll.refreshComplete');
                        $ionicScrollDelegate.resize();
                        $ionicScrollDelegate.scrollTop(true);
                    }
                });
            };

            var myPopup;
            $scope.show_offer = function () {

                $scope.offer_seen = true;

                // Add animate css class to out popup container
                $timeout(function () {
                    var popupElements = document.getElementsByClassName("popup-container");
                    if (popupElements.length) {
                        $scope.popupElement = angular.element(popupElements[0]);
                        $scope.popupElement.addClass('animated');
                        $scope.popupElement.addClass('bounceInDown');
                    }
                }, 1);

                myPopup = $ionicPopup.show({
                    templateUrl: 'templates/offers.html',
                    title: $scope.provider.offer.title,
                    scope: $scope
                });
                IonicClosePopupService.register(myPopup);

                try
                {
                    ga_storage._trackPageview('show_offer.html', 'Show Offer');
                }
                catch (e)
                {
                    console.log(e.message);
                }
            };

            $scope.close_offers = function () {
                myPopup.close();
            };
        });