angular.module('getlaundry.app_services', [])

        .factory('Badge', function ($cordovaBadge) {
            return {
                set: function (count) {
                    $cordovaBadge.hasPermission().then(function (yes) {
                        $cordovaBadge.set(count).then(function () {
                            // You have permission, badge set.
                        }, function (err) {
                            // You do not have permission.
                        });
                    }, function (no) {
                        // You do not have permission
                    });
                },
                clear: function () {
                    $cordovaBadge.hasPermission().then(function (yes) {
                        $cordovaBadge.clear().then(function () {
                            // You have permission, badge cleared.
                        }, function (err) {
                            // You do not have permission.
                        });
                    }, function (no) {
                        // You do not have permission
                    });
                }
            };
        })

        .factory('Message', function ($ionicPopup, Loading, $cordovaDialogs, IonicClosePopupService, $timeout) {
            return {
                show: function (title, message, close) {
                    Loading.hide();
                    console.log(title + ': ' + message);
                    if (false && ionic.Platform.isIOS()) {
                        $cordovaDialogs.alert(message, title, 'Close')
                                .then(function () {
                                });
                    }
                    else {
                        // Add animate css class to out popup container
                        $timeout(function () {
                            var popupElements = document.getElementsByClassName("popup-container");
                            if (popupElements.length) {
                                popupElement = angular.element(popupElements[0]);
                                popupElement.addClass('animated');
                                popupElement.addClass('bounceInDown');
                            }
                        }, 1);
                        var popup = $ionicPopup.alert({
                            title: title,
                            template: message
                        });
                        IonicClosePopupService.register(popup);
                    }
                },
                alert: function (title, message) {
                    var popup = $ionicPopup.alert({
                        title: title,
                        template: message
                    });
                },
                confirm: function (title, message) {
                    if (false && ionic.Platform.isIOS()) {
                        return $cordovaDialogs.confirm(message, title, ['Close', 'Confirm']);
                    }
                    else {
                        var confirmPopup = $ionicPopup.confirm({
                            title: title,
                            template: message,
                            cancelType: 'assertive-border'
                        });
                        return confirmPopup;
                    }
                }
            };
        })

        .factory('Session', function () {
            return {
                get: function (key) {
                    return window.localStorage.getItem('laundry_' + key);
                },
                set: function (key, val) {
                    return window.localStorage.setItem('laundry_' + key, val);
                },
                unset: function (key) {
                    return window.localStorage.removeItem('laundry_' + key);
                }
            };
        })

        .factory('Auth', function (Session) {
            return {
                logout: function () {
                    Session.unset('access');
                    Session.unset('api_signature');
                    Session.unset('customer');
                    Session.unset('providers');
                    Session.unset('provider');
                    Session.unset('order');
                    Session.unset('cart');
                    Session.unset('id_address');
                    Session.unset('is_pickup');
                    Session.unset('address');
                    Session.unset('location');
                    Session.unset('currency');
                    Session.unset('push_asked');
                    Session.unset('redirect');
                    Session.unset('previous_view');
                    Session.unset('address_details');
                    Session.unset('loaded_address');
                    Session.unset('loaded_details');
                    Session.unset('editing_order');
                    Session.unset('from_map');
                    if (typeof intercom != 'undefined') {
                        intercom.reset();
                        intercom.registerUnidentifiedUser();
                        console.log('reset intercom');
                    }
                },
                isLogged: function () {
                    if (Session.get('access') == 'OK' && Session.get('api_signature') != ''
                            && Session.get('api_signature') != 'null' && Session.get('api_signature') != null
                            && Session.get('api_signature') != 'undefined') {
                        return true;
                    }

                    return false;
                }
            };
        })

        .factory('Loading', function ($ionicLoading) {
            return {
                show: function (message, duration) {
                    $ionicLoading.show({
                        duration: typeof duration != 'undefined' ? duration : 15000,
                        template: message
                    });
                },
                hide: function () {
                    $ionicLoading.hide();
                }
            };
        })

        .factory('Intercom', function (FCustomer, APP_VERSION) {
            return {
                init: function () {
                    var customer = FCustomer.getCustomer();
                    if (typeof customer != 'undefined' && customer) {
                        if (typeof intercom != 'undefined') {
                            intercom.registerIdentifiedUser({email: customer.email});
                            intercom.updateUser({
                                custom_attributes: {
                                    name: customer.firstname,
                                    user_id: customer.id_customer,
                                    language: customer.language,
                                    contact: customer.contact,
                                    country_signup: customer.country_signup,
                                    app_version: APP_VERSION,
                                    gender: customer.gender
                                }
                            });
                            console.log('intercom init');
                        }
                    }
                }
            };
        })

        .factory('Push', function (API_URL, APP_VERSION, Session, FCustomer, $http, Message, $state, Badge, Auth, $cordovaDevice) {

            return {
                register: function () {

                    if (typeof PushNotification == 'undefined') {
                        return true;
                    }

                    var push = PushNotification.init({
                        android: {
                            senderID: "339818665715",
                            sound: "true",
                            vibrate: "true"
                        },
                        ios: {
                            alert: "true",
                            badge: "true",
                            sound: "true",
                            vibrate: "true"
                        },
                        windows: {}
                    });
                    push.on('registration', function (data) {

                        if (data.registrationId.length > 0) {

                            var gcmData = {
                                register_id: data.registrationId,
                                platform: device.platform,
                                app_version: APP_VERSION,
                                device: $cordovaDevice.getDevice(),
                                api_signature: Session.get('api_signature')
                            };
                            $http.post(API_URL + 'register_gcm', gcmData)
                                    .success(function (res) {
                                        if (res.success) {
                                            window.localStorage['registered'] = 'ok';
                                        }
                                    })
                                    .error(function (err) {
                                        //Message.show(err);
                                    });
                        }
                    });
                    push.on('notification', function (data) {

                        // data.message,
                        // data.title,
                        // data.count,
                        // data.sound,
                        // data.image,
                        // data.additionalData

                        if (data.message) {

                            Message.show(data.message);
                            var id_order = data.additionalData.id_order;
                            if (Auth.isLogged() && typeof id_order != 'undefined') {
                                $state.go('app.view_order', {id_order: id_order}, {reload: true});
                            }
                        }

                        if (data.sound) {
                            var snd = new Media(data.sound);
                            snd.play();
                        }

                        if (data.count) {
                            Badge.set(data.count);
                        }
                    });
                    push.on('error', function (e) {
                        //alert(e.message);
                    });
                },
                remove: function () {

                    if (typeof PushNotification != 'undefined') {
                        push = PushNotification.init({
                            android: {
                                senderID: "339818665715"
                            },
                            ios: {
                                alert: "true",
                                badge: "true",
                                sound: "true"
                            },
                            windows: {}
                        });
                        push.unregister(function () {
                            console.log('unsubscribed success');
                        }, function () {
                            console.log('unsubscribed error');
                        });
                    }
                }
            };
        })

        .factory('Language', function ($translate, $cordovaSplashscreen, $window, Message, Session) {
            //add the languages you support here. ar stands for arabic

            var allLanguages = ['en', 'ar', 'ae', 'qa', 'lb', 'th', 'my', 'id'];
            var rtlLanguages = ['ae', 'ar', 'qa', 'lb'];
            var isRtl = function (languageKey) {

                if (typeof languageKey == 'undefined' || languageKey == null) {
                    languageKey = $translate.proposedLanguage() || $translate.use();
                }
                for (var i = 0; i < rtlLanguages.length; i += 1) {
                    if (languageKey.indexOf(rtlLanguages[i]) > -1)
                        return true;
                }
                return false;
            };
            //public api
            return {
                isRtl: isRtl,
                setLanguage: function (lang) {

                    if (allLanguages.indexOf(lang) == -1) {
                        Message.show('Translation for this language is coming soon...');
                        return false;
                    }

                    if (lang == 'ar' || lang == 'qa' || lang == 'lb') {
                        lang = 'ae';
                    }

                    Session.set('language', lang);
                    if ((this.isRtl() && !this.isRtl(lang)) || (!this.isRtl() && this.isRtl(lang))) {
                        console.log('Reloading...');
                        $cordovaSplashscreen.show();
                        $window.location.reload(true);
                        return false;
                    }
                    else {
                        $translate.use(lang);
                    }
                },
                getLanguage: function () {
                    return $translate.preferredLanguage();
                }
            };
        })

        .factory('Rate', function ($translate, Session) {
            return {
                rateApp: function () {
                    AppRate.preferences.useLanguage = Session.get('language');
                    var popupInfo = {};
                    popupInfo.title = $translate.instant('rate_title');
                    popupInfo.message = $translate.instant('rate_message');
                    popupInfo.cancelButtonLabel = $translate.instant('rate_cancel');
                    popupInfo.laterButtonLabel = $translate.instant('rate_later');
                    popupInfo.rateButtonLabel = $translate.instant('rate_now');
                    AppRate.preferences.customLocale = popupInfo;
                    AppRate.preferences.openStoreInApp = true;
                    AppRate.preferences.storeAppURL.ios = '980937537';
                    AppRate.preferences.storeAppURL.android = 'market://details?id=com.builtbysf.getlaundrymobile';
                    AppRate.promptForRating(true);
                }
            };
        })

        .factory('Common', function ($http, Session, API_URL) {
            return {
                get_faq: function () {
                    return $http.post(API_URL + 'get_faq', {
                        api_signature: Session.get('api_signature')
                    });
                },
                get_terms: function () {
                    return $http.post(API_URL + 'get_terms', {
                        api_signature: Session.get('api_signature')
                    });
                },
                business_request: function (provider) {
                    return $http.post(API_URL + 'business_request', {
                        api_signature: Session.get('api_signature'),
                        provider: provider
                    });
                }
            };
        })

        .factory('FConnection', function ($cordovaNetwork, $rootScope, $state) {

            function checkModules(only_check) {
                if (typeof CheckoutKit == 'undefined') {
                    loadCheckoutKit();

                    if (only_check == 1) {
                        return false;
                    }
                }

                if (typeof google == "undefined" || typeof google.maps == "undefined") {
                    loadGoogleMaps();
                }
                else {
                    $state.go('app.map');
                }
            }

            function loadGoogleMaps() {

                var exists = document.getElementById("googleMaps");

                if (typeof exists == 'undefined') {
                    var apiKey = 'AIzaSyC3ZePFDbV1g3-qdx357WqxkONr-Ppd-R0';
                    var script = document.createElement("script");
                    script.type = "text/javascript";
                    script.id = "googleMaps";
                    script.src = 'http://maps.google.com/maps/api/js?key=' + apiKey + '&libraries=places';
                    document.body.appendChild(script);

                    console.log('Loaded map SDK');
                }
            }

            function loadCheckoutKit() {

                var script = document.createElement("script");
                script.type = "text/javascript";
                script.id = "CheckoutKit";
                script.src = 'https://cdn.checkout.com/js/checkoutkit.js';
                document.body.appendChild(script);

                console.log('Loaded CheckoutKit SDK');
            }

            return {
                isOnline: function () {

                    if (ionic.Platform.isWebView()) {
                        return $cordovaNetwork.isOnline();
                    }
                    else {
                        return navigator.onLine;
                    }
                },
                isOffline: function () {

                    if (ionic.Platform.isWebView()) {
                        return !$cordovaNetwork.isOnline();
                    }
                    else {
                        return !navigator.onLine;
                    }
                },
                listen: function () {
                    if (ionic.Platform.isWebView()) {

                        $rootScope.$on('$cordovaNetwork:online', function (event, networkState) {
                            console.log('online');
                            checkModules();
                        });
                        $rootScope.$on('$cordovaNetwork:offline', function (event, networkState) {
                            console.log('offline');
                            $state.go('app.internet_error');
                        });
                    }
                    else {
                        window.addEventListener("online", function (e) {
                            checkModules();
                        }, false);
                        window.addEventListener("offline", function (e) {
                            $state.go('app.internet_error');
                        }, false);
                    }
                },
                reload_modules: function (only_check) {
                    checkModules(only_check);
                }
            };
        });


